#ifndef _SORTED_ARRAY_DICTIONARY_H_
#define _SORTED_ARRAY_DICTIONARY_H_

#include "abstract_dictionary.h"
#include "sorted_array_list.h"

// KeyType must be partially ordered
// ValueType must support copy 
template <typename KeyType, typename ValueType>
class SortedArrayDictionary: public AbstractDictionary<KeyType, ValueType>
{
public:

    // return true if dict is empty, else false
    bool isEmpty() const;

    // return number of key-value pairs in the dict
    std::size_t size() const;

    // add an item to the dict
    // throws std::bad_alloc
    void add(const KeyType& key, const ValueType& value);

    // remove key-value from the dict
    // throws std::logic_error if dict is empty
    void remove(const KeyType& key);

    // remove all key-values from the dict
    void clear();

    // get a copy of value assocaited with key
    // throws std::logic_error if entry does not exist
    ValueType get(const KeyType& key) const;

    // return true if dict contains key
    bool contains(const KeyType& key) const;

private:

    //TODO
    
};

#include "sorted_array_dictionary.txx"

#endif // _SORTED_ARRAY_DICTIONARY_H_
